 //--------------------------------------------------------------------------------------------------
 // Copyright (c) 2011,2012 FZ Software
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without modification, are permitted
 // provided that the following conditions are met:
 //
 // Redistributions of source code must retain the above copyright notice, this list of
 // conditions and the following disclaimer.
 //
 // Redistributions in binary form must reproduce the above copyright notice, this list of
 // conditions and the following disclaimer in the documentation and/or other materials provided with
 // the distribution.
 // Neither the name of this project nor the names of its contributors may be used to endorse
 // or promote products derived from this software without specific prior written permission.
 //
 // THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR
 // IMPLIED WARRANTIES, INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTIBILITY AND
 // FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 // CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 // DAMAGES (INCLUDING BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 // DATA OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 // IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 // OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 //
 // this is simplified version of code to run the battery charger circuit using 12F683
 // for public release
 //--------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------
// $Name:$
// $Id: batchgr.h,v 1.9.2.1 2012/06/25 05:03:42 admin Exp $
// $Date: 2012/06/25 05:03:42 $
// $Revision: 1.9.2.1 $
// $State: Exp $
// $Source: /Development/cvs/projects/BatChgr-1x/batchgr.h,v $
//-------------------------------------------------------------------------------

#define VDD 5
#define _EEADR 0x2100
#define _EEADDR_StartChar  0
#define _EEADDR_DeviceAddress 1
#define _EEADDR_DeviceType    3
#define _EEADDR_DeviceVersion 4

#define _EEADDR_VOLTSREF		10
#define _EEADDR_VOLTRANGE		12
#define _EEADDR_VOLTSINMAX		14
#define _EEADDR_VOLTSINMAX_VREF 16

#define _EEADDR_CELL_NIMH_PARMS 32
#define  _EEADDR_CELL_PARAMETER_BLOCKS _EEADDR_CELL_NIMH_PARMS

/////////////////////////////////////////////////////////////////////////
// cell charge parameter blocks
#define ECELL_TYPE 0						// byte
#define ECELL_VOLTAGE 2					// 2byte
#define ECELL_OPERATING_VOLTAGE_LOW  4  // normal discharged cell voltage
#define ECELL_OPERATING_VOLTAGE_HIGH 6  // normal charged cell voltage
#define ECELL_VOLTAGE_CRITICAL_LOW   8  // critical low battery, dead ?
													 // below this value - pulse charge
#define ECELL_CRITICAL_LOW_CHRG_PULSE_VOLTAGE 10
#define ECELL_CRITICAL_LOW_CHRG_MAXTIME 12 // minutes to attempt revive of critcal low charge
#define ECELL_CRITICAL_LOW_PULSERATE 14	 // number of pulses per minute

#define ECELL_VOLTAGE_FOR_LOW_CHARGE_PULSE   16 // below this value, use pulse charge
#define ECELL_LOW_CHARGE_PULSE_MAXTIME       18 // max time for pulse charge stage in minutes
#define ECELL_LOW_CHARGE_PULSERATE				20

#define ECELL_VOLTAGE_CHARGE_BULK   22  // battery voltage to start  bulk charge

#define ECELL_CHARGE_BULK_MAXTIME	24  // max time for a bulk charge
#define ECELL_CHARGE_BULK_VOLTAGE 	26  // ideal charging voltage for bulk charge

#define ECELL_EOC_VOLTAGE 			28 // End-of-Charge voltage
#define ECELL_EOC_PULSE_VOLTAGE  30 // EOC voltage for pulse charge
#define ECELL_EOC_PULSERATE		32
#define ECELL_EOC_PULSE_MAXTIME  34 // minutes for pulse charge

#define ECELL_OVER_CHARGE			36 // immediate switch off voltage
#define ECELL_EOC_DT					38 // end-of-charge temperature difference to ambient
#define ECELL_EOC_DV					40 // end-of-charge delta-v voltaqe
#define ECELL_EOC_DV_TIME			42 // max time minutes charge at dv stage

#define ECELL_FLOAT_PULSERATE 	44
#define ECELL_FLOAT_PULSE_VOLTAGE 46

#define ECELL_MAX_TEMPERATURE    48
#define ECELL_CHARGE_CUTOFF_TIME    50 // minutes
#define ECELL_TEMPERATURE_START_BULKCHARGE_TRESHOLD 52
#define ECELL_PARAMETER_BLOCK_SIZE 54
///////////
#define _EEADDR_CELL_NIMH_PARMS_2 (_EEADDR_CELL_PARAMETER_BLOCKS+ECELL_PARAMETER_BLOCK_SIZE)

///////////
#define _EEADDR_CREF_TABLE _EEADDR_CELL_NIMH_PARMS+ECELL_PARAMETER_BLOCK_SIZE+1
// Flag1 bits
#define Flag_Tick 0
#define Flag_Seconds 1
#define Flag_Minutes 2
#define Flag_SerialRx 6

////////////////////////////////////////////////
enum eBatteryChem
{
	BAT_UNKNOWN=0,
	BAT_LION=1,
	BAT_NIMH
};

enum eBatteryState
{
   CHRG_UNKN=0,
   CHRG_CRITICAL_LOW,
	CHRG_BULK,
	CHRG_EOC_CONSTANT_V,
	CHRG_PULSEFLOAT
};

enum eChargingState
{
	CHARGING_NONE=0,
	BATTERY_DEAD,
	CRITICAL_LOW_RECOVERY,
	LOW_CHARGING_PULSE,
	BULK_CHARGING,
	EOC_CHARGING_PULSE,
	CHARGE_COMPLETED,
	FLOAT_CHARGING
};


enum eChargerState
	{
		S_NO_OP  = -1,
		S_IDLE	=0,	// idling
		S_WAIT,
		S_WAIT_LED_FLASH,
		S_WAIT_LED_FLASH_S2,
		S_STARTUP,
		S_STARTUP_1,
		S_STARTUP_2,
		S_STARTUP_3,
		S_STARTUP_4,
		S_STARTUP_5,

		S_BATVOLTMEASURE	= 16,
		S_BATVOLTMEASURE_S1,
		S_BATVOLTMEASURE_S2,
		S_BATVOLTMEASURE_S3,
		S_BATVOLTMEASURE_S4,
		S_BATVOLTMEASURE_S5,
		S_BATVOLTMEASURE_S6,
		S_BATVOLTMEASURE_S7,
		S_BATVOLTMEASURE_S8,
		S_BATVOLTMEASURE_S9,
		S_BATVOLTMEASURE_S10,
		S_BATVOLTMEASURE_S11,
		S_BATVOLTMEASURE_S12,
		S_BATVOLTMEASURE_S13,

		S_BATVOLTDECISION,

		S_TEMPMEASURE	=32,
		S_TEMPMEASURE_S1,
		S_TEMPMEASURE_S2,
		S_TEMPMEASURE_S3,

		S_BATDETECTED ,		// found voltage on battery terminal

		S_CHARGE_START,
		S_CHARGE_START_S1,
		S_CHARGE_START_S2,


		S_CRITICAL_LOW_RECOVERY	= 42,
		S_CRITICAL_LOW_RECOVERY_S1,
		S_CRITICAL_LOW_RECOVERY_S2,
		S_CRITICAL_LOW_RECOVERY_S3,
		S_CRITICAL_LOW_RECOVERY_S4,
		S_CRITICAL_LOW_RECOVERY_S5,


		S_BATTBULKCHARGE=60,
		S_BATTBULKCHARGE_S1,
		S_BATTBULKCHARGE_S2,
		S_BATTBULKCHARGE_S3,
		S_BATTBULKCHARGE_S4,




		S_BATTPULSEFLOAT=80,
		S_BATTPULSEFLOAT_S1,
		S_BATTPULSEFLOAT_S2,
		S_BATTPULSEFLOAT_S3,
		S_BATTPULSEFLOAT_S4,
		S_BATTPULSEFLOAT_S5,
		S_BATTPULSEFLOAT_S6,

		S_CHARGE_COMPLETE=90,
		S_CHARGE_COMPLETE_S1,
		S_CHARGE_COMPLETE_S2,
		S_CHARGE_COMPLETE_S3,
		S_CHARGE_ABORT,

		S_BATDISCONNECT =100,		// battery terminal voltage gone to Zero
		S_BATDISCONNECTED,
		S_BATCHECK , 			// start check mode
		S_BATCHECK_STG2,			// measure voltage
										// pulse charge for 10 seconds
										// measure voltage
										// if voltage increased, ok to charge 1
										// otherwise fully charged or dead
		S_BATCHECK_STG4,			//
		S_BATMAXCHARGE,			// charge at maximum power

		S_SOURCEVOLTMEASURE,
		S_SOURCEVOLTMEASURE_S1,
		S_SOURCEVOLTMEASURE_S2,


		S_SUPPLYLOW,				// power supply got low
		S_CHARGING_EOC=120,
		S_CHARGE_FAIL,
		S_CHARGE_FAIL_S2,
		S_CHARGE_FAIL_S3,
		S_CHARGE_FAIL_S4,

		S_CHARGE_STOPPED

	};

enum eErrorRecord {
	r_ERROR_CLEARED=0,
	r_CHARGE_COMPLETED_TEMPERATURE_LIMIT=1,
	r_CHARGE_COMPLETED_TEMPERATURE_DIFF,
	r_CHARGE_COMPLETED_DELTAVOLT,
	r_ECELL_OVER_CHARGE,
	r_ECELL_END_OF_CHARGE,
	r_CHARGE_TIME_LIMIT,
	r_CHARGE_TIME_LIMIT_CRITICAL_LOW_RECOVERY,
	r_CHARGE_TIME_LIMIT_LOW_CHARGING_PULSE,

};
////////////////////////////////////////////////////////////////
void OnTick();
void OnSecond();





void Hex2BinHi(char c);
void Hex2BinLo(char c);

////////////////
// voltage divider resistors is 3.9k + 1k
// R1 + R2
// Vm is the divided measured voltage
// VREF is the on-chip Voltage Reference
#define VREF (4.95)
// Vin = Vref * 11
// Vm  = Vin / 11
// Vin(max) = Vref * 11 = 33.96V
//
// Vin is measured as a fraction of Vin(max)
//
//#define VinMAX ((((VREF*11)*10)+5)/10)
#define VDivR1 1.5
#define VDivR2 2.0
#define VinMAX ((((VREF*((VDivR1+VDivR2)/VDivR2))*10)+5)/10)
//#define VinNormed(Vin)   ((((((Vin*VREF)/VinMAX)*1024)*10)+5)/10)
//reduce.....
#define VinNormed(Vin)     (((((Vin*VDivR2*1024)/(VDivR1+VDivR2))*10)+5)/10)
//
// R1 1k, R2 10K, measured XFactor....
//#define XFactor	(1.125)
#define XFactor	(1.1)

#define VmREFNormed      ((((VREF*1024)*10)+5)/10)
//#define VinREFNormed(Vin) ((((((((Vin*1024)*VDivR2)/(VDivR1+VDivR2)))/VREF)*10)+5)/10)
#define VinREFNormed(Vin) ((((Vin*1024)/(XFactor*VREF)*10)+5)/10)
#define V_Normed(V)	((((V*1024)*10)+5)/10)
#define fract(dec,fraction) (((dec*100+fraction)*128+50)/100)
//#define VmNormed(
#define VOLTRANGE fract(33,96)

#define VOLTS(volt) (((((volt)*1000)/VOLTRANGE)*1023)/1000)
#define VOLTSDB(volt) (((uns16)volt)&0x00ff),((((uns16)volt)>>8)&0x00ff)
#define VOLTSLOBYT(volt) (((uns16)volt)&0x00ff)
#define INTEGER16(val) (val&0x00ff),(val>>8)&0x00ff
#define EEINT(num) (((uns16)num)&0x00ff),((((uns16)num)>>8)&0x00ff)

#define f2voltage(f) ((long)f* (long)((((VOLTRANGE)*2000L)/1023)+5) )/2000)
// no. of tick overs per minute defines the pulse rate period
#define PERMINUTE_TICKS(n) EEINT((((1000/TickerMillisec)*60)/n)/2)

#define PULSEWIDTH_TICKS 1

#define TEMPERATURE_START 509
#define CVREF1406 1

#define LEDFLASH_ON_SHORT_TICKLENGTH 24
#define LEDFLASH_ON_LONG_TICKLENGTH 70
#define LEDFLASH_ON_TICKLENGTH 60
#define LEDFLASH_OFF_TICKLENGTH 80
#define LEDFLASH_OFF_LONG_TICKLENGTH 120

