 //--------------------------------------------------------------------------------------------------
 // Copyright (c) 2011,2012 FZ Software
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without modification, are permitted
 // provided that the following conditions are met:
 //
 // Redistributions of source code must retain the above copyright notice, this list of
 // conditions and the following disclaimer.
 //
 // Redistributions in binary form must reproduce the above copyright notice, this list of
 // conditions and the following disclaimer in the documentation and/or other materials provided with
 // the distribution.
 // Neither the name of this project nor the names of its contributors may be used to endorse
 // or promote products derived from this software without specific prior written permission.
 //
 // THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR
 // IMPLIED WARRANTIES, INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTIBILITY AND
 // FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 // CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 // DAMAGES (INCLUDING BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 // DATA OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 // IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 // OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 //
 // this is simplified version of code to run the battery charger circuit using 12F683
 // for public release
 //--------------------------------------------------------------------------------------------------
//#pragma chip PIC16F629
//#pragma chip PIC12F683
//#include <12F683.h>

   #include "hexcodes.h"
   #include "_12F629.h"

#define  CALVALUE  136
#define  APROCFREQ 8000000
#define  Ticker_TIMER USE_TMR0
//#define  WDPSD 128
//#define  UseWD
#define  TickerMillisec 10
#define  TickerMilliTick 10
#define T0PSD 8


#include "PicStd.h"
#include "batchgr.h"

//-------------------------------------------------------------------------------
// $Name:$
// $Id: batchgr_f683.c,v 1.14.2.1 2012/06/25 05:09:53 admin Exp $
// $Date: 2012/06/25 05:09:53 $
// $Revision: 1.14.2.1 $
// $State: Exp $
// $Source: /Development/cvs/projects/BatChgr-1x/batchgr_f683.c,v $
//-------------------------------------------------------------------------------
// 1/2 cell battery charger using switch mode
// 	will pulse charge a dead battery for x minutes to check if a charge
//		develops
//		will bulk charge
//		end-of-charge will pulse charge
//  	floating pulse charge
//
//
//  Uses Comparator and Voltage Reference Internal to PIC12F629
//  Comp Mode CM2:CM:0 = 100
//
//           PIC12F683
//
//           +------+
//      VDD -+1    8+- VSS
//      GP5 -+2    7+- GP0     CIN+
//      GP4 -+3    6+- GP1     CIN-
//      GP3 -+4    5+- GP2     COUT
//           +------+
//
//
//  Pin Description:
//  GP2  COUT driving base of PNP switching TR
//  GP1  F/B voltage to Comparator, Battery current to ADC
//  GP0  Battery voltage to ADC
//  GP5  LED Indicator
//
//  GP4  Thermistor voltage AN3
//
//  GP3  Input button resets charge time override
//
//
//  Activity Description:
//
//  Circuit voltage VSUPPLY is supplied from always ON 12V
//
//	 if Battery Voltage > ECELL_OVER_CHARGE
//		 charge complete
//
//	 if was charging and Battery Voltage > ECELL_EOC_VOLTAGE
//		 ECELL_EOC_PULSERATE
//		 ECELL_EOC_PULSE_MAXTIME
//
//  if Battery Voltage < ECELL_VOLTAGE_CRITICAL_LOW
//		 pulse charge and wait for rising voltage (charge)
//		 ECELL_CRITICAL_LOW_PULSERATE
//		 if time > maxtime
//			fail charge
//	 if Battery Voltage < ECELL_VOLTAGE_FOR_LOW_CHARGE_PULSE
//		 pulse charge and wait for rising voltage (charge)
//		 ECELL_LOW_CHARGE_PULSERATE
//		 if time > maxtime
//			fail charge
//
//  else bulk rate charge
//
//	 if charging and Temperatur > ECELL_MAX_TEMPERATURE
//		  charge complete
//
//
//--------------------------------------------------------------------
#ifdef PIC12F629
#define GOTCALIBRATE 1

#define CP_OFF |=(1<<7)
#define CPD_OFF |=(1<<8)
#endif
#ifdef PIC12F683
#define CP_OFF |=(1<<6)
#define CPD_OFF |=(1<<7)
#endif

#pragma config  WDTE=on,PWRTE=on,BODEN=on,FOSC=4,CP_OFF,CPD_OFF
//#pragma config |= (1<<5) // MCLR

//#pragma config |= (1<<12)|(1<<8)|(1<<7)
//#pragma config = (1<<6)|(1<<5)|(1<<4)|(1<<3)|4
//*********************************************************
#ifdef PIC12F629
 #define CMCON0 CMCON
#endif
/* assign names to port pins */

bit bRB0 @ GPIO.0;
bit bRB1 @ GPIO.1;
bit bRB2 @ GPIO.2;
bit bRB3 @ GPIO.3;
bit bRB4 @ GPIO.4;
bit bRB5 @ GPIO.5;
bit bTMR1ON @ T1CON.0;
bit bCOUT @ CMCON0.6;
bit bT1IE @ PIE1.0;

bit bRBCOUT @ GPIO.2;
bit bRBLED  @ GPIO.5;
bit bTRIS_RBLED @ TRISIO.5;

bit bTRIS_RB0 @ TRISIO.0;
bit bTRIS_RB1 @ TRISIO.1;
bit bTRIS_RB5 @ TRISIO.5;

bit bTRIS_SERIALRX @ TRISIO.2;

#define ADBitNo_BattVoltage 0
#define ADBitNo_BattCurrent 1
#define ADBitNo_Thermistor 4
#define ADCHN_BattVoltage 0
#define ADCHN_BattCurrent 1
#define ADCHN_Thermistor 3
#define COUT_BitNo 2
#define LEDBitNo 5
#define ButtonBitNo 3



//-------------------------

/* global variables */
#pragma cdata[0] = __NOP

#ifdef PIC12F629
#pragma cdata[1] =   __CALL(ADDRESS_LAST_CODE_LOCATION)
#endif

// #pragma cdata[2] =  __MOVWF(OSCCAL)
//#endasm
#pragma resetVector 2    // change to address 1

#include "int16CXX.H"

//#define CalValueConstant 1
#pragma rambank 0
uns8 Flag1;
bit  bADgo;
bit  bChargingPulse;
bit  bChargingOn;
bit  bWaitTickCounter;
bit  bSecCounter;
bit  bPinSecondsPulse;
bit  bLED1Flash;
bit  bLED1Flash_on;
bit  bLEDShortFlash;
bit  bTemperatureStart;
bit  bBrownOutOccured;
bit  bBatteryDisconnected;
bit  bCommandButton;

uns8 TickT0Counter;
uns8 SecCounter;

//*---Interrupt----------------------------------------
#pragma origin 4

interrupt int_server( void)
{
    int_save_registers    // W, STATUS (and PCLATH)


  if (T0IF)  {
        /* TMR0 overflow interrupt */
       T0IF = 0;  /* reset flag */
       TickT0Counter++;
	    if(TickT0Counter==(TICKCOUNTS))
       {
           TickT0Counter=0;
           Flag1 |= (1<<Flag_Tick);

		     SecCounter++;
     	     if(SecCounter == TICKSFORSECOND)
       	  {
             Flag1 |= (1<<Flag_Seconds);
             SecCounter=0;
       	  }
        }

	}

	if(GPIF) {
		W=GPIO;	// only have IOC on GPIO.3
		if( (W & (1<<ButtonBitNo) )==0)
			bCommandButton=1;
		GPIF=0;
	}


   /*
     NOTE: GIE is AUTOMATICALLY cleared on interrupt entry and set
           to 1 on exit (by RETFIE). Setting GIE to 1 inside the
           interrupt service routine will cause nested interrupts
           if an interrupt is pending. Too deep nesting may crash
           the program !
   */

    int_restore_registers; // W, STATUS (and PCLATH)
}
//*----------------------------------------------------------------
#include "eelib.h"
/////////////////////////////////////////
#pragma rambank 0
//uns16 MyTickCounter;
uns8 MySecCounter;
uns16 WaitTickCounter;
uns8 LEDFlashTickCounter;
uns8 LEDFlashCounter;
uns8 MinuteCounter;
int8 vrefset;
uns8 nCells;
uns8 mstate,mstate_next;
uns8 mstate_temp;
uns8 mstate_stack[3];
uns16 wait_stack[3];
uns8 mstate_stack_ptr;
uns8 mstate_next_onSecond;
uns8 mstate_next_onSecond_next;
uns8 mstate_next_onTick;
uns8 error_record;

uns8 mcharging_state;
//uns8 mstate_check;
uns8 mbattery_state;

uns16 adBatVolts;
bank1 uns16 adThermistor;

uns8  Vi;
uns16 Vmax;
uns16 Vmin;

uns16 VBat_volts;			// recording of battery voltage
uns16 VBat_state_start; // record of battery voltage at start of current charge state
uns16 VBat_peak;			// recorded battery peak voltage
int16 VBat_diff;			// difference between peak voltage and current
uns16 pulse_timing;

bank1 uns16 maxlimit_stagetime; // maximum time in minutes allowed for current charge state
bank1 uns16 MinuteCount_stage;  // counting the minutes for the current charge stage

bank1 uns16 temperature_limit;  // temperature limit
bank1 uns16 start_temperature;  // initial temperature recorded
bank1 uns16 temperature_last;   // previous temperature
bank1 int16 temperature_diff;   // dT
bank1 uns16 MinuteCount_Charging; // total time in minutes for charging
bank1 uns16 sleep_periods;
//uns16 MinuteCount_EOC_Charging;



//#define setTickCounter(n,nextState)  {MyTickCounter=n;bTickCounter=1;mstate_next_onTick=nextState;}
#define setSecCounter(n,nextState) 	{MySecCounter=n;bSecCounter=1;mstate_next_onSecond=nextState;mstate_next_onSecond_next=S_NO_OP;}
#define setSecCounter2(n,nextState,nextnextState) 	{MySecCounter=n;bSecCounter=1;mstate_next_onSecond=nextState;mstate_next_onSecond_next=nextnextState;}

#define clearTickCounter()	{bWaitTickCounter=0;}
#define clearTickerStack() {mstate_stack_ptr=0;}
#define clearSecCounter() 	{bSecCounter=0;mstate_next_onSecond=S_NO_OP;}

#define start_idle() {mstate++;mstate_next=mstate;mstate=S_IDLE;}
#define LEDON()			 {bRBLED=0;bLED1Flash=0;}
#define LEDOFF()			 {bRBLED=1;bLED1Flash=0;}

#define FLASHLEDSHORT(n) {;bLED1Flash=1;bLED1Flash_on=0;bLEDShortFlash=1;LEDFlashCounter=n;}
#define FLASHLEDLONG(n)  {;bLED1Flash=1;bLED1Flash_on=0;bLEDShortFlash=0;LEDFlashCounter=n;}
#define SLEEPIDLE(n) {mstate=S_IDLE;sleep_periods=n;}
//#define WaitLEDFLASH() {mstate++;mstate_next=mstate;mstate=S_WAIT_LED_FLASH;}
/////////////////////////////////////////
void OnTick();
void OnSecond();
void OnMinute();
void OnADdone();
void battery_decision();
void batteries_number();
void battery_charge_state(uns8 nextChargingState);

void push_state(uns8 state)
{
	mstate_stack[mstate_stack_ptr]=state;
	mstate_stack_ptr++;
}
uns8 pop_state()
{
	mstate_stack_ptr--;
	if(mstate_stack_ptr==0)
		bWaitTickCounter=0;
	else {
		uns16 t;
		WaitTickCounter=wait_stack[mstate_stack_ptr-1];
		t = wait_stack[mstate_stack_ptr];
		if(WaitTickCounter > t ){
			WaitTickCounter=WaitTickCounter - t;
		}else {
			WaitTickCounter=1;
		}
	}
	return mstate_stack[mstate_stack_ptr];
}
// current wait timer is [mstate_stack_ptr-1]
//
void push_wait_state(uns16 n,uns8 state)
{
	mstate_stack[mstate_stack_ptr]=state;
	if(mstate_stack_ptr>0) {
		uns16 t,t2;
		t = wait_stack[mstate_stack_ptr-1];
		t = t - WaitTickCounter;
		t2 = 	wait_stack[mstate_stack_ptr-1];
		t2 = t2 -t;
	 	wait_stack[mstate_stack_ptr-1] = t2;
	}
	wait_stack[mstate_stack_ptr]=n;
	mstate_stack_ptr++;
}
// wait_Ticks
// start a locked wait loop
//
// timer could already be active....
//
void wait_Ticks(uns8 n)
{
	mstate++;
	push_wait_state((uns16)n,mstate);
	WaitTickCounter=n;
	bWaitTickCounter=1;
	mstate=S_WAIT;
}

void setOnTicksCounter(uns16 n, uns8 onTickState)
{
	bWaitTickCounter=1;
	push_wait_state(n, onTickState);
	WaitTickCounter=n;
}

void WaitLEDFLASH() {
	mstate++;mstate_next=mstate;mstate=S_WAIT_LED_FLASH;
}

void set_onSeconds(uns8 n, uns8 onSecondsState)
{
	MySecCounter=n;
	bSecCounter=1;
	mstate_next_onSecond=onSecondsState;
}

void wait_us(uns8 n)
{
	while(--n){
#if APROCFREQ==32000000
	 nop();nop();nop();nop();nop();nop();nop();nop();
#elif APROCFREQ==8000000
	 nop();nop();
#elif APROCFREQ==4000000
	 nop();
#endif
 }
}
//
// setup the comparator to act as switch regulator
// n
// 7 0.73A
// 8 0.84A
// 9 0.94A
//10 1.04A
//11 1.14A
//12 1.24A
//13 1.33A heatsink starting to warm
//14 1.43A
//15
void set_battery_charging(uns8 n)
{
	// When VRR = 1: CVREF = (VR3:VR0 / 24) * VDD
	// When VRR = 0: CVREF = VDD/4 + (VR3:VR0 / 32) * VDD
	VRCON = (1<<nVREN)|(0<<nVRR)| n;

   ANSEL &= ~((1<<ADBitNo_BattVoltage)|(1<<COUT_BitNo));

    /* Comparator with Output and Internal Reference
     * the output COUT drives base of a series pass switching PNP transistor
     * voltage f/b to CIN- (pin
     */
	CMCON0 =  (0<<nCINV) |(0<<nCIS) |  0b011;

}


void set_charging_on()
{
	bChargingOn =1;
	CMCON0 =  (0<<nCINV) |(0<<nCIS) |  0b011;
}

void set_charging_off()
{
	CMCON0 = 0x07; //Comparator Off (Lowest power)
	bRBCOUT=0;
	bChargingOn =0;
}

void set_measure_battery_volts()
{
	set_charging_off();
	TRISIO |= (1<<ADBitNo_BattVoltage);
	ANSEL = (0b0.1101<<nADCS0)|(1<<ADBitNo_BattVoltage); //FOSC/64
	ADCON0 = (ADCHN_BattVoltage<<nADCHN)|(0<<nVCFG)|(1<<nADFM)|(1<<nADON);
	wait_Ticks(200); //wait 500ms before measuring
}


void init1()
{
	VBat_peak=0;
	nCells=1;
	bTemperatureStart=0;
	if(bBrownOutOccured==0||bBatteryDisconnected) {
		mcharging_state=CHARGING_NONE;
		MinuteCount_Charging=0;
		MinuteCount_stage=0;
		mstate_stack_ptr=0;
		temperature_last=-1;
		bTemperatureStart=1;
		bBatteryDisconnected=0;
	}
	bADgo=0;
   bSecCounter=0;
	bWaitTickCounter=0;
	error_record=0;
	LEDOFF();
}
////////////////////////////////////////////////////////////////////////////
// main()
////////////////////////////////////////////////////////////////////////////
void main( void)
{
 #ifdef GOTCALIBRATE
   OSCCAL = W;
 #else
 #ifdef CalValueConstant
    OSCCAL = CALVALUE;
 #else
 //   #asm
  //   DW __CALL(0x3ff);
  //  #endasm
//    OSCCAL = W;
 #endif
 #endif

#ifdef PIC12F683
 OSCCON = 0b0111.0001;
 WDTCON = (10<<1)|1;
#endif

OPTION_REG = OPTION_GPPU | OPTION_INTEDG | OPTION_T0CS | OPTION_T0SE | OPTION_PSA | OPTION_PS20;
CMCON0 = 0x07; //Comparator Off (Lowest power)

TRISIO = (0<<LEDBitNo)|(0<<COUT_BitNo)|(1<<ADBitNo_BattVoltage)|(1<<ADBitNo_Thermistor)|(1<<ADBitNo_BattCurrent);
GPIO = 0b.0000.0000;  /* out = 1 */
IOC = (1<<ButtonBitNo);
WPU = 0b.0000.0000;
W = GPIO; /* clear mismatch */
RBIF = 0;  /* reset flag */
bPinSecondsPulse=1;
LEDOFF();
//VRset=1;
mstate=S_STARTUP;

if(POR_==0) {
	BOD_=1;
	POR_=1;
	bBrownOutOccured=0;
}else {
	if(BOD_==0) { // brown out occured
	  BOD_=1; // set again
	  bBrownOutOccured=1;
	}
}
    /* main is terminated by a SLEEP instruction */
//set_battery_charging( (1<<nVRR)|3);
set_charging_off();

INTCON = (1<<nGIE)|(1<<nT0IE)|(0<<nPEIE)|(1<<nGPIE);


do{
  clrwdt();




  if(bADgo && ADGO==0)
  {
		OnADdone();
  }


  { // user loop

	if(bCommandButton) {
		bCommandButton=0;
		mstate=S_STARTUP;
	}

	switch(mstate){
	case S_NO_OP:
							break;
	case S_IDLE:
							GIE=0;
							do {
							sleep();
							sleep_periods--;
							if(bCommandButton)
								break; //sleep loop
							}while(sleep_periods>0);
							GIE=1;
							if(mstate_next!=S_NO_OP)
							{
								mstate=mstate_next;
								mstate_next=S_NO_OP;
							}
							else {
								mstate=S_STARTUP;
								bTemperatureStart=1;
							}
							break;
	case S_WAIT:
							break;

	case S_WAIT_LED_FLASH:
							if(LEDFlashCounter==0) {
								wait_Ticks(10);
							}
							break;

	case S_WAIT_LED_FLASH_S2:
							mstate++;
							if(mstate_next!=S_NO_OP)
							{
								mstate=mstate_next;
								mstate_next=S_NO_OP;
							}
							break;
   case S_STARTUP:
							init1();
							FLASHLEDSHORT(5);
							wait_Ticks(200);
							break;
	case S_STARTUP_1:
							mstate=S_TEMPMEASURE;
							mstate_next=S_STARTUP_2;
					//		setOnTicksCounter(700,S_STARTUP_3);
					//		wait_Ticks(200);
							break;
	case S_STARTUP_2:
							if(bBrownOutOccured==0) {
								start_temperature=adThermistor;
							}
							mstate=S_BATVOLTMEASURE;
							mstate_next=S_STARTUP_3;
							break;
	case S_STARTUP_3:
							batteries_number();
						//	setSecCounter(3,S_STARTUP_5);
						//	setSecCounter(4,S_STARTUP_5);
							WaitLEDFLASH();
							break;
	case S_STARTUP_4:
//	case S_STARTUP_5:
							battery_charge_state(CHARGING_NONE);
							if(nCells!=0){
								mstate=S_BATVOLTDECISION;
							} else  {
								SLEEPIDLE(30);
							}
							break;


	case S_BATVOLTMEASURE:
							set_measure_battery_volts();
							Vi=1;
							break;
	case S_BATVOLTMEASURE_S1: // wait delay
							wait_Ticks(120);
							break;
	case S_BATVOLTMEASURE_S2: // wait delay
							ADGO=1;
							bADgo=1;
							mstate++;
							break;
	case S_BATVOLTMEASURE_S3: // wait ad conversion
							break;
	case S_BATVOLTMEASURE_S4: // get the battery measurement and repeat until a stable value
							mstate=mstate+Vi;
							break;
	case S_BATVOLTMEASURE_S5: // get the battery measurement and repeat until a stable value
							Vmax=Vmin=adBatVolts;
							Vi++;
							mstate=S_BATVOLTMEASURE_S1;
							break;
	case S_BATVOLTMEASURE_S6: // get the battery measurement and repeat until a stable value
	case S_BATVOLTMEASURE_S7: // get the battery measurement and repeat until a stable value
	case S_BATVOLTMEASURE_S8: // get the battery measurement and repeat until a stable value
	case S_BATVOLTMEASURE_S9: // get the battery measurement and repeat until a stable value
	case S_BATVOLTMEASURE_S10: // get the battery measurement and repeat until a stable value

							if(adBatVolts > Vmax)
								Vmax=adBatVolts;
							if(adBatVolts < Vmin)
								Vmin=adBatVolts;
							Vi++;
							mstate=S_BATVOLTMEASURE_S1;
							break;

	case S_BATVOLTMEASURE_S11: // get the battery measurement and repeat until a stable value

							{
								int16 t;
								t = Vmax - Vmin;
								if(t <= 3)
									mstate++;
								else
									mstate=S_BATVOLTMEASURE;
							}
							break;

	case S_BATVOLTMEASURE_S12:	// measurement complete
							VBat_volts=Vmin;
							mstate++;
							break;
	case S_BATVOLTMEASURE_S13: //got conversion
							//battery_charge_state();
							mstate++; //S_BATVOLTDECISION
							if(mstate_next!=S_NO_OP)
							{
								mstate=mstate_next;
								mstate_next=S_NO_OP;
							}
							break;
	case S_BATVOLTDECISION: //got conversion
							battery_decision();
							break;
	case S_TEMPMEASURE:
							TRISIO |= (1<<ADBitNo_Thermistor);
							ANSEL = (0b0.110<<nADCS0)|(1<<ADBitNo_Thermistor); //FOSC/32
							ADCON0 = (ADCHN_Thermistor<<nADCHN)|(0<<nVCFG)|(1<<nADFM)|(1<<nADON);
							wait_Ticks(1);
							break;
	case S_TEMPMEASURE_S1:
							ADGO=1;
							bADgo=1;
							mstate++;
							break;
	case S_TEMPMEASURE_S2:
							break;

	case S_TEMPMEASURE_S3:
							if(!bTemperatureStart) {
								temperature_limit=ReadEEDataInt(_EEADDR_CELL_NIMH_PARMS+ECELL_MAX_TEMPERATURE);
								if(adThermistor >= temperature_limit) {
									error_record=r_CHARGE_COMPLETED_TEMPERATURE_LIMIT;
									battery_charge_state(CHARGE_COMPLETED);;
									break;
								}
								temperature_diff = adThermistor -temperature_last;
								temperature_limit=ReadEEDataInt(_EEADDR_CELL_NIMH_PARMS+ECELL_EOC_DT);
								if(temperature_diff > (signed)temperature_limit)
								{
									error_record=r_CHARGE_COMPLETED_TEMPERATURE_DIFF;
									battery_charge_state(CHARGE_COMPLETED);
									break;
								}
							} else {
								bTemperatureStart=0;
							}
							temperature_last=adThermistor;

							if(mstate_next!=S_NO_OP){
								mstate=mstate_next;
								mstate_next=S_NO_OP;
							}
							else mstate=S_STARTUP;
							break;

	case  S_CRITICAL_LOW_RECOVERY:
							set_battery_charging( (1<<nVRR)|0x0F);
							pulse_timing=ReadEEDataInt(_EEADDR_CELL_NIMH_PARMS+ECELL_CRITICAL_LOW_PULSERATE);
							setSecCounter(20,S_BATVOLTMEASURE);
							mstate++;
							break;
	case  S_CRITICAL_LOW_RECOVERY_S1:// pulse wait time
							set_charging_off();
							setOnTicksCounter(pulse_timing,S_CRITICAL_LOW_RECOVERY_S3);
							mstate++;
							FLASHLEDSHORT(1);
							battery_charge_state(CRITICAL_LOW_RECOVERY);
							break;
	case  S_CRITICAL_LOW_RECOVERY_S2:// pulse wait time
							break;
	case  S_CRITICAL_LOW_RECOVERY_S3:// pulse width time
							set_charging_on();
							setOnTicksCounter(30,S_CRITICAL_LOW_RECOVERY_S1);
							mstate++;
							break;
	case  S_CRITICAL_LOW_RECOVERY_S4:
							break;
	case  S_CRITICAL_LOW_RECOVERY_S5:
							mstate=S_CRITICAL_LOW_RECOVERY;
							break;


	case  S_BATTBULKCHARGE:
							LEDON();
							set_battery_charging( (1<<nVRR)|0x08); //0.84Amp
							setSecCounter2(60,S_BATVOLTMEASURE,S_BATTBULKCHARGE_S4);
							mstate++;
							battery_charge_state(BULK_CHARGING);
							break;
	case  S_BATTBULKCHARGE_S1:
							mstate_next=mstate+1;
							mstate=S_TEMPMEASURE;
							break;
	case  S_BATTBULKCHARGE_S2:
							//just wait for seconds timer //mstate=S_BATTBULKCHARGE;
							break;
	case  S_BATTBULKCHARGE_S3:
							break;
	case  S_BATTBULKCHARGE_S4:
							//just wait for seconds timer //mstate=S_BATTBULKCHARGE;
							mstate=S_BATTBULKCHARGE;
							battery_decision();
							break;


	case S_BATTPULSEFLOAT:
							clearTickCounter();
							setSecCounter(60,S_BATVOLTMEASURE);
							pulse_timing=ReadEEDataInt(_EEADDR_CELL_NIMH_PARMS+ECELL_FLOAT_PULSERATE);
							set_battery_charging( (1<<nVRR)|0x0A);
							mstate++;
							break;
	case S_BATTPULSEFLOAT_S1:
							set_charging_off();
							FLASHLEDLONG(1);
							setOnTicksCounter(pulse_timing,S_BATTPULSEFLOAT_S3);
							mstate++;
							break;
	case S_BATTPULSEFLOAT_S2:
							break;
	case S_BATTPULSEFLOAT_S3:
							set_charging_on();
							setOnTicksCounter(25,S_BATTPULSEFLOAT_S1);
							mstate++;
							break;
	case S_BATTPULSEFLOAT_S4:
							break;


	case S_CHARGE_COMPLETE:
							clearTickCounter();
							set_charging_off();
							FLASHLEDLONG(2);
							setSecCounter(10,S_CHARGE_COMPLETE_S2);
							mstate++;
							break;
	case S_CHARGE_COMPLETE_S1:
							break;
	case S_CHARGE_COMPLETE_S2:
							mstate=S_BATVOLTMEASURE;
							mstate_next=S_CHARGE_COMPLETE;
							if(adBatVolts == 0) { // battery removed
								SLEEPIDLE(300);
							}
							if(MinuteCount_stage > 40) {
								battery_charge_state(FLOAT_CHARGING);
							}
							break;

	case S_CHARGE_ABORT:

	case S_CHARGE_FAIL:
							clearTickCounter();
							clearSecCounter();
							set_charging_off();
							FLASHLEDLONG(2);
							WaitLEDFLASH();
							break;
	case S_CHARGE_FAIL_S2:
//	case S_CHARGE_FAIL_S3:
							FLASHLEDSHORT(error_record);
							setSecCounter(10,S_CHARGE_FAIL);
							mstate=S_BATVOLTMEASURE;
							mstate_next=S_CHARGE_FAIL_S4;
							break;
	case S_CHARGE_FAIL_S4:
							if(adBatVolts == 0) { // battery removed
								SLEEPIDLE(300);
							}
							break;
	default:
							break;
  }
 }/*end user loop*/


  if(Flag1& (1<<Flag_Tick))
  {
     Flag1 &= ~(1<<Flag_Tick);
     OnTick();
  }
  else if(Flag1 & (1<<Flag_Seconds))	// seconds is lower priority
  {
     Flag1 &= ~(1<<Flag_Seconds);
     OnSecond();
     MinuteCounter++;
     if(MinuteCounter==60)
  	  {
			MinuteCounter=0;
			OnMinute();
  	  }
  }

 } while ( 1);  /* forever  loop */

}

void OnTick()
{

  if(bWaitTickCounter )
  {
	 if(WaitTickCounter==0)
	 {
	 	mstate=pop_state();
	 }
	 WaitTickCounter--;
  }

  LEDFlashTickCounter--;
  if(LEDFlashTickCounter==0) {

	if(bLED1Flash) {
		 if(bLED1Flash_on) {
			bLED1Flash_on=0;
			bRBLED=1; // off
			if(bLEDShortFlash) {
				LEDFlashTickCounter=LEDFLASH_OFF_TICKLENGTH;
			}
			else  {
				LEDFlashTickCounter=LEDFLASH_OFF_LONG_TICKLENGTH;
			}
		   LEDFlashCounter--;
			if(LEDFlashCounter==0) {
				bLED1Flash=0;
			}
		 }
		 else if(bLED1Flash_on==0) {
			bLED1Flash_on=1;
			bRBLED=0; // on
			if(bLEDShortFlash) {
				LEDFlashTickCounter=LEDFLASH_ON_SHORT_TICKLENGTH;
			}
			else  {
				LEDFlashTickCounter=LEDFLASH_ON_LONG_TICKLENGTH;
			}
		}
	}
	else {
		LEDFlashTickCounter=LEDFLASH_ON_TICKLENGTH;
	}
  }
}


void OnSecond()
{
	MySecCounter--;
	if(bSecCounter && MySecCounter==0)
	{
	 bSecCounter=0;
	 if(mstate_next_onSecond!=S_NO_OP) {
		if(mstate_next_onSecond_next!=S_NO_OP) {
			mstate_next=mstate_next_onSecond_next;
		}
		mstate=mstate_next_onSecond;
		mstate_next_onSecond=S_NO_OP;
		mstate_next_onSecond_next=S_NO_OP;
	 } else {
			mstate++;
			mstate_next=S_NO_OP;
	 }
	}
}

void OnMinute()
{
	MinuteCount_stage++;
}

void OnADdone()
{
   uns16 adVal;

	adVal.high8= ADRESH;
	adVal.low8 = ADRESL;
	bADgo=0;
	switch(mstate) {
	case S_BATVOLTMEASURE_S3:
					adBatVolts = adVal;
					mstate++;
					break;
	case S_TEMPMEASURE_S2:
					adThermistor = adVal;
					mstate++;
					break;
	}
}

void battery_decision()
{
	uns16 stagetime;
	int16 volts;
	int16 temp @ volts;
	uns8  parmblock;

	parmblock=_EEADDR_CELL_NIMH_PARMS;
	if(nCells==2)
		parmblock=_EEADDR_CELL_NIMH_PARMS_2;


	if(VBat_volts > VBat_peak)
		VBat_peak=VBat_volts;

	VBat_diff = VBat_peak - VBat_volts  ;

	volts = ReadEEDataInt(parmblock+ECELL_EOC_DV);
	if(VBat_diff > volts)
	{
		error_record=r_CHARGE_COMPLETED_DELTAVOLT;
		battery_charge_state(CHARGE_COMPLETED);
		return;
	}


   volts = ReadEEDataInt(parmblock+ECELL_OVER_CHARGE);
	if(VBat_volts >= volts) {
		error_record=r_ECELL_OVER_CHARGE;
		battery_charge_state(CHARGE_COMPLETED);
		return;
	}

   volts = ReadEEDataInt(parmblock+ECELL_EOC_VOLTAGE);
	if(VBat_volts > volts)
	{
		error_record=r_ECELL_END_OF_CHARGE;
		battery_charge_state(CHARGE_COMPLETED);
		return;
	}
	if(mcharging_state==CHARGING_NONE)
	{
   	volts = ReadEEDataInt(parmblock+ECELL_OPERATING_VOLTAGE_HIGH);
		if(VBat_volts > volts)
		{
			battery_charge_state(BULK_CHARGING);
		//	maxlimit_stagetime=maxlimit_stagetime/2;
			return;
		}

	}

	volts = ReadEEDataInt(parmblock+ECELL_VOLTAGE_CHARGE_BULK);
	if(VBat_volts >= volts)
	{
		battery_charge_state(BULK_CHARGING);
		return;
	}

	volts = ReadEEDataInt(parmblock+ECELL_VOLTAGE_CRITICAL_LOW);
	if(VBat_volts <= volts)
	{

		if(mcharging_state == CHARGING_NONE) {
			battery_charge_state(CRITICAL_LOW_RECOVERY);
			return;
		}

		if(mcharging_state == CRITICAL_LOW_RECOVERY) {
			battery_charge_state(CRITICAL_LOW_RECOVERY);
		}
		mstate=S_CHARGE_ABORT;
	}

	mstate=mstate_next;
}

//
// determine if 1 cell or 2 cell in series
void batteries_number()
{
	uns16 volts;
	nCells=1;
	volts = ReadEEDataInt(_EEADDR_CELL_NIMH_PARMS_2+ECELL_OPERATING_VOLTAGE_LOW);
	if(VBat_volts >= volts)
	{
		nCells=2;
	}
	if(VBat_volts==0)
	{
		nCells=0;
		bBatteryDisconnected=1;
		return;
	}
	FLASHLEDLONG(nCells);
}




void battery_charge_state(uns8 nextChargingState)
{
	int16 volts_diff;
	if(mcharging_state!=nextChargingState)
	{
		switch(nextChargingState) {
		case CHARGING_NONE:
					MinuteCount_Charging=0;
					break;
		case CRITICAL_LOW_RECOVERY:
					mstate=S_CRITICAL_LOW_RECOVERY;
					maxlimit_stagetime = ReadEEDataInt(_EEADDR_CELL_NIMH_PARMS+ECELL_EOC_PULSE_MAXTIME);
					break;

		case BULK_CHARGING:
					mstate=S_BATTBULKCHARGE;
					maxlimit_stagetime = ReadEEDataInt(_EEADDR_CELL_NIMH_PARMS+ECELL_CHARGE_BULK_MAXTIME);
					break;

		case FLOAT_CHARGING:
					mstate=S_BATTPULSEFLOAT;
					break;
		case CHARGE_COMPLETED:
					LEDOFF();
					mstate=S_CHARGE_COMPLETE;
					mcharging_state=CHARGE_COMPLETED;
					return;
		}
		clearTickCounter();
		clearSecCounter();
		VBat_state_start=VBat_volts;
		MinuteCount_stage=0;
		mcharging_state=nextChargingState;
		return;
	}

	if(MinuteCount_stage >= maxlimit_stagetime)
	{
		error_record=r_CHARGE_TIME_LIMIT;

		switch(mcharging_state){
		case CHARGING_NONE:
					break;
		case CRITICAL_LOW_RECOVERY:
					volts_diff=VBat_volts - VBat_state_start;
					volts_diff = volts_diff - VinREFNormed(0.25);
					if( volts_diff >= 0)
							mstate=S_BATTBULKCHARGE;
					else {
							mstate=S_CHARGE_FAIL;
							error_record=r_CHARGE_TIME_LIMIT_CRITICAL_LOW_RECOVERY;
					}
					break;
		case BULK_CHARGING:
					mstate=S_CHARGE_ABORT;
					break;

		case FLOAT_CHARGING:
					break;
		}
	}

}
//////////////////////////////////////

//#define D_OSCCAL 136
//#pragma origin ADDRESS_LAST_CODE_LOCATION
//
//void LoadOSCCAL()
//{
//#asm
// OVRet:
//		 	retlw D_OSCCAL
//#endasm
//}
//#pragma origin ADDRESS_LAST_CODE_LOCATION
#include "BatChgr_eedata.c"



// When VRR = 1: CVREF = (VR3:VR0 / 24) * VDD
// When VRR = 0: CVREF = VDD/4 + (VR3:VR0 / 32) * VDD
//
//  VRR  ref   VDD    CVREF
//    0    0   5.0    1.25
//    0    1   5.0 	 1.406
//    0    2   5.0	 1.563
//    0    3   5.0	 1.719
//    0    4   5.0  	 1.875
//    0    5   5.0	 2.031
//    0    6   5.0	 2.188
//    0    7   5.0	 2.344
//    0    8   5.0	 2.50
//    0    9   5.0	 2.656
//    0   10   5.0	 2.813
//    0   11   5.0	 2.969
//    0   12   5.0	 3.125
//    0   13   5.0	 3.281
//    0   14   5.0	 3.438
//    0   15   5.0	 3.594
//
//    1    1   5.0 	 0.208
//    1    2   5.0	 0.416
//    1    3   5.0	 0.625
//    1    4   5.0	 0.833
//    1    5   5.0	 1.042
//    1    6   5.0	 1.25
//    1    7   5.0	 1.458
//    1    8   5.0	 1.667
//    1    9   5.0	 1.875
//    1   10   5.0	 2.083
//    1   11   5.0	 2.29
//    1   12   5.0	 2.5
//    1   13   5.0	 2.71
//    1   14   5.0	 2.92
//    1   15   5.0	 3.125
/////////////////////////////////